package graph;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.Random;

import javax.swing.JPanel;

/**
 * Komponenta uživatelského rozhraní vykreslující koláčový graf s
 * popiskami hodnot.
 */
public class GraphComponent extends JPanel {

    /**
     * Hodnoty zobrazované v koláčovém grafu.
     */
    private int[] values;
    /**
     * Barvy jednotlivých výsečí koláčového grafu.
     */
    private Color[] colors;

    /**
     * Vytvoří novou komponentu koláčového grafu.
     *
     * @param values
     *            pole zobrazovaných hodnot.
     */
    public GraphComponent(int[] values) {
        this.values = values;
        if (values != null) {
            Random generator = new Random();
            colors = new Color[values.length];
            for (int i = 0; i < colors.length; i++) {
                colors[i] = new Color(generator.nextInt() & 0xFFFFFF);
            }
        }
        setBackground(new Color(187, 187, 170));
    }

    /**
     * Pomocná metoda, která vrátí nejvhodnější barvu pro dané
     * pozadí. Pokud je pozadí světlé, vrátí černou barvu. Pokud je
     * pozadí tmavé, vrátí bílou barvu.
     *
     * @param background
     *            barva pozadí.
     * @return nejvhodnější barva textu pro danou barvu pozadí.
     */
    private Color getTextColorForBackground(Color background) {
        double y =
                0.3 * background.getRed() + 0.59
                * background.getGreen() + 0.11
                * background.getBlue();
        if (y > 127) {
            return Color.BLACK;
        } else {
            return Color.WHITE;
        }
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        int size = getWidth();
        if (size > getHeight()) {
            size = getHeight();
        }
        int x = (getWidth() - size) / 2;
        int y = (getHeight() - size) / 2;

        double angle = 0;

        int sum = 0;

        g.setFont(new Font("serif", Font.BOLD, 16));


        if (values.length == 0) {
          g.drawString("Neplatné hodnoty v session.", 20, getHeight()/2);
        }
             else{
                for (int i = 0; i < values.length; i++) {
                sum += values[i];
            }
            for (int i = 0; i < values.length; i++) {
                g.setColor(colors[i]);
                double value = (values[i] * 360.0) / sum;
                int valueInt = (int) Math.round(value);
                int angleInt = (int) Math.round(angle);
                g.fillArc(x, y, size, size, angleInt, valueInt);
                g.setColor(getTextColorForBackground(g.getColor()));
                drawText(g, "" + values[i], angleInt + valueInt / 2,
                        size / 2 - 20, x + size / 2, y + size / 2);
                angle += value;
            }
        }
    }

    /**
     * Pomocná metoda, která vykreslí text na pozici dané polárními
     * souřadnicemi. Souřadnice určují střed textu.
     *
     * @param g
     *            grafický kontext, pomocí kterého se kreslí.
     * @param text
     *            kreslený text.
     * @param angle
     *            polární souřadnice - úhel.
     * @param r
     *            polární souřadnice - poloměr.
     * @param cX
     *            souřadnice středu.
     * @param cY
     *            souřadnice středu.
     */
    private static void drawText(Graphics g, String text, int angle,
            int r, int cX, int cY) {
        int centerX =
                (int) Math.round(r * Math.cos(Math.PI * angle / 180));
        int centerY =
                (int) -Math.round(r
                * Math.sin(Math.PI * angle / 180));
        FontMetrics fm = g.getFontMetrics();
        int leftX = centerX - fm.stringWidth(text) / 2;
        int baselineY =
                centerY - fm.getHeight() / 2 + fm.getLeading()
                + fm.getAscent();
        g.drawString(text, cX + leftX, cY + baselineY);
    }
}
